
/* Chrysalide - Outil d'analyse de fichiers binaires
 * demanglers.c - équivalent Python du fichier "core/demanglers.c"
 *
 * Copyright (C) 2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "demanglers.h"


#include <pygobject.h>


#include <core/demanglers.h>


#include "../helpers.h"



/* Fournit le décodeur de désignations correspondant à un type. */
static PyObject *py_demanglers_get_for_type(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = NULL car méthode statique.                            *
*                key = nom technique du décodeur recherché.                   *
*                                                                             *
*  Description : Fournit le décodeur de désignations correspondant à un type. *
*                                                                             *
*  Retour      : Décodeur trouvé et mis en place ou None.                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_demanglers_get_for_type(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Désignation à retourner     */
    const char *key;                        /* Nom court du format         */
    int ret;                                /* Bilan de lecture des args.  */
    GCompDemangler *demangler;              /* Décodeur mis en place       */

    ret = PyArg_ParseTuple(args, "s", &key);
    if (!ret) return NULL;

    demangler = get_compiler_demangler_for_type(key);

    if (demangler != NULL)
    {
        result = pygobject_new(G_OBJECT(demangler));
        Py_INCREF(result);

        g_object_unref(G_OBJECT(demangler));

    }
    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_demanglers_type(void)
{
    static PyMethodDef py_demanglers_methods[] = {

        { "get_for_type", py_demanglers_get_for_type,
          METH_VARARGS | METH_STATIC,
          "get_for_type(key, /)\n--\n\nCreate a new demangler for a given type of encoding."
        },
        { NULL }

    };

    static PyTypeObject py_demanglers_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name = "pychrysalide.core.demanglers",
        .tp_basicsize = sizeof(PyObject),

        .tp_flags = Py_TPFLAGS_DEFAULT,

        .tp_doc = "Access to the code demanglers",

        .tp_methods = py_demanglers_methods

    };

    return &py_demanglers_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.core.demanglers'.      *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_demanglers(PyObject *module)
{
    PyTypeObject *py_demanglers_type;       /* Type Python de 'demanglers' */
    int ret;                                /* Bilan d'un appel            */

    py_demanglers_type = get_python_demanglers_type();

    py_demanglers_type->tp_new = PyType_GenericNew;

    if (PyType_Ready(py_demanglers_type) != 0)
        return false;

    Py_INCREF(py_demanglers_type);
    ret = PyModule_AddObject(module, "demanglers", (PyObject *)py_demanglers_type);

    return (ret == 0);

}
