
/* Chrysalide - Outil d'analyse de fichiers binaires
 * dwarf_def.h - liste des constantes utilisées par le format DWARF
 *
 * Copyright (C) 2008-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _FORMAT_DWARF_DWARF_DEF_H
#define _FORMAT_DWARF_DWARF_DEF_H



#include <stdint.h>


#include "../../arch/vmpa.h"
#include "../../common/leb128.h"




/**
 * § 7.5.1.1 Compilation Unit Header.
 */

/* En-tête présente dans certaines sections */
typedef struct _dw_compil_unit_header
{
    uint64_t unit_length;                   /* Taille totale sans le champ */
    bool is_32b;                            /* Le format est-il sur 32b ?  */

    uint16_t version;                       /* Version de la section       */

    uint64_t debug_abbrev_offset;           /* Emplacement dans la section */
    uint8_t address_size;                   /* Taille des adresses         */

} dw_compil_unit_header;


/**
 * § 7.5.4 Attribute Encodings
 */

/* Figure 18, Tag encodings, begins here. */

typedef enum _DwarfTag
{
    DW_TAG_array_type               = 0x01,
    DW_TAG_class_type               = 0x02,
    DW_TAG_entry_point              = 0x03,
    DW_TAG_enumeration_type         = 0x04,
    DW_TAG_formal_parameter         = 0x05,
    DW_TAG_imported_declaration     = 0x08,
    DW_TAG_label                    = 0x0a,
    DW_TAG_lexical_block            = 0x0b,
    DW_TAG_member                   = 0x0d,
    DW_TAG_pointer_type             = 0x0f,
    DW_TAG_reference_type           = 0x10,
    DW_TAG_compile_unit             = 0x11,
    DW_TAG_string_type              = 0x12,
    DW_TAG_structure_type           = 0x13,
    DW_TAG_subroutine_type          = 0x15,
    DW_TAG_typedef                  = 0x16,
    DW_TAG_union_type               = 0x17,
    DW_TAG_unspecified_parameters   = 0x18,
    DW_TAG_variant                  = 0x19,
    DW_TAG_common_block             = 0x1a,
    DW_TAG_common_inclusion         = 0x1b,
    DW_TAG_inheritance              = 0x1c,
    DW_TAG_inlined_subroutine       = 0x1d,
    DW_TAG_module                   = 0x1e,
    DW_TAG_ptr_to_member_type       = 0x1f,
    DW_TAG_set_type                 = 0x20,
    DW_TAG_subrange_type            = 0x21,
    DW_TAG_with_stmt                = 0x22,
    DW_TAG_access_declaration       = 0x23,
    DW_TAG_base_type                = 0x24,
    DW_TAG_catch_block              = 0x25,
    DW_TAG_const_type               = 0x26,
    DW_TAG_constant                 = 0x27,
    DW_TAG_enumerator               = 0x28,
    DW_TAG_file_type                = 0x29,
    DW_TAG_friend                   = 0x2a,
    DW_TAG_namelist                 = 0x2b,
    DW_TAG_namelist_item            = 0x2c,
    DW_TAG_packed_type              = 0x2d,
    DW_TAG_subprogram               = 0x2e,
    DW_TAG_template_type_parameter  = 0x2f,
    DW_TAG_template_value_parameter = 0x30,
    DW_TAG_thrown_type              = 0x31,
    DW_TAG_try_block                = 0x32,
    DW_TAG_variant_part             = 0x33,
    DW_TAG_variable                 = 0x34,
    DW_TAG_volatile_type            = 0x35,

    DW_TAG_dwarf_procedure          = 0x36,
    DW_TAG_restrict_type            = 0x37,
    DW_TAG_interface_type           = 0x38,
    DW_TAG_namespace                = 0x39,
    DW_TAG_imported_module          = 0x3a,
    DW_TAG_unspecified_type         = 0x3b,
    DW_TAG_partial_unit             = 0x3c,
    DW_TAG_imported_unit            = 0x3d,
    DW_TAG_condition                = 0x3f,
    DW_TAG_shared_type              = 0x40,

    /* Nouveautés v.4 */

    DW_TAG_type_unit                = 0x41,
    DW_TAG_rvalue_reference_type    = 0x42,
    DW_TAG_template_alias           = 0x43,

    DW_TAG_lo_user                  = 0x4080,
    DW_TAG_hi_user                  = 0xffff

} DwarfTag;

/* Figure 19. Child determination encodings. */

#define DW_CHILDREN_no 0x00
#define DW_CHILDREN_yes 0x01

/* Figure 21. Attribute form encodings. */

typedef enum _DwarfForm
{
    DW_FORM_invalid     = 0x00,

    DW_FORM_addr        = 0x01,
    DW_FORM_block2      = 0x03,
    DW_FORM_block4      = 0x04,
    DW_FORM_data2       = 0x05,
    DW_FORM_data4       = 0x06,
    DW_FORM_data8       = 0x07,
    DW_FORM_string      = 0x08,
    DW_FORM_block       = 0x09,
    DW_FORM_block1      = 0x0a,
    DW_FORM_data1       = 0x0b,
    DW_FORM_flag        = 0x0c,
    DW_FORM_sdata       = 0x0d,
    DW_FORM_strp        = 0x0e,
    DW_FORM_udata       = 0x0f,
    DW_FORM_ref_addr    = 0x10,
    DW_FORM_ref1        = 0x11,
    DW_FORM_ref2        = 0x12,
    DW_FORM_ref4        = 0x13,
    DW_FORM_ref8        = 0x14,
    DW_FORM_ref_udata   = 0x15,
    DW_FORM_indirect    = 0x16,

    /* Nouveautés v4 */

    DW_FORM_sec_offset  = 0x17,
    DW_FORM_exprloc     = 0x18,
    DW_FORM_flag_present = 0x19,
    DW_FORM_ref_sig8    = 0x20

} DwarfForm;






/* Résumé des différentes valeurs dans DWARF v2 */
typedef union _dw_v2_form_value
{
    /* DW_FORM_addr */
    virt_t address;

    /* DW_FORM_data[1248] */
    uint8_t data1;
    uint16_t data2;
    uint32_t data4;
    uint64_t data8;

    /* DW_FORM_sdata */
    leb128_t sdata;

    /* DW_FORM_udata */
    uleb128_t udata;

    /* DW_FORM_block[124]? */
    struct
    {
        const bin_t *start;
        phys_t size;

    } block;

    /* DW_FORM_string */
    /* DW_FORM_strp */
    const char *string;

    /* DW_FORM_flag */
    uint8_t flag;

    /* DW_FORM_ref[1248] */
    uint8_t ref1;
    uint16_t ref2;
    uint32_t ref4;
    uint64_t ref8;

    /* DW_FORM_ref_udata */
    uleb128_t ref_udata;

} dw_v2_form_value;


/* Résumé des différentes valeurs dans DWARF v4 */
typedef union _dw_v4_form_value
{
    dw_v2_form_value v2;

    /* DW_FORM_sec_offset */
    uint64_t sec_offset;

    /* DW_FORM_exprloc */
    struct
    {
        const bin_t *start;
        phys_t size;

    } expr;

    /* DW_FORM_flag_present */
    bool has_flag;

    /* DW_FORM_ref_sig8 */
    uint64_t signature;

} dw_v4_form_value;


/* Résumé des différentes valeurs dans DWARF */
typedef union _dw_form_value
{
    dw_v2_form_value v2;
    dw_v2_form_value v4;

} dw_form_value;



















/**
 * § 7.5.3 Abbreviations Tables.
 */

/* Description d'un attribut d'une abréviation */
typedef struct _dw_abbrev_raw_attr
{
    uleb128_t name;                         /* Désignation de l'attribut   */
    uleb128_t form;                         /* Nature de sa valeur         */

} dw_abbrev_raw_attr;

/* En-tête de déclaration d'une abréviation */
typedef struct _dw_abbrev_decl
{
    uleb128_t code;                         /* Identifiant attribué        */
    uleb128_t tag;                          /* Identifiant attribué        */

    uint8_t has_children;                   /* Présence de sous-entrées ?  */

} dw_abbrev_decl;










/* Liste des attributs rencontrés */
typedef enum _DwarfAttrib
{
    DW_AT_sibling               = 0x01,
    DW_AT_location              = 0x02,
    DW_AT_name                  = 0x03,
    DW_AT_ordering              = 0x09,
    DW_AT_byte_size             = 0x0b,
    DW_AT_bit_offset            = 0x0c,
    DW_AT_bit_size              = 0x0d,
    DW_AT_stmt_list             = 0x10,
    DW_AT_low_pc                = 0x11,
    DW_AT_high_pc               = 0x12,
    DW_AT_language              = 0x13,
    DW_AT_discr                 = 0x15,
    DW_AT_discr_value           = 0x16,
    DW_AT_visibility            = 0x17,
    DW_AT_import                = 0x18,
    DW_AT_string_length         = 0x19,
    DW_AT_common_reference      = 0x1a,
    DW_AT_comp_dir              = 0x1b,
    DW_AT_const_value           = 0x1c,
    DW_AT_containing_type       = 0x1d,
    DW_AT_default_value         = 0x1e,
    DW_AT_inline                = 0x20,
    DW_AT_is_optional           = 0x21,
    DW_AT_lower_bound           = 0x22,
    DW_AT_producer              = 0x25,
    DW_AT_prototyped            = 0x27,
    DW_AT_return_addr           = 0x2a,
    DW_AT_start_scope           = 0x2c,
    DW_AT_bit_stride            = 0x2e,
    DW_AT_upper_bound           = 0x2f,
    DW_AT_abstract_origin       = 0x31,
    DW_AT_accessibility         = 0x32,
    DW_AT_address_class         = 0x33,
    DW_AT_artificial            = 0x34,
    DW_AT_base_types            = 0x35,
    DW_AT_calling_convention    = 0x36,
    DW_AT_count                 = 0x37,
    DW_AT_data_member_location  = 0x38,
    DW_AT_decl_column           = 0x39,
    DW_AT_decl_file             = 0x3a,
    DW_AT_decl_line             = 0x3b,
    DW_AT_declaration           = 0x3c,
    DW_AT_discr_list            = 0x3d,
    DW_AT_encoding              = 0x3e,
    DW_AT_external              = 0x3f,
    DW_AT_frame_base            = 0x40,
    DW_AT_friend                = 0x41,
    DW_AT_identifier_case       = 0x42,
    DW_AT_macro_info            = 0x43,
    DW_AT_namelist_item         = 0x44,
    DW_AT_priority              = 0x45,
    DW_AT_segment               = 0x46,
    DW_AT_specification         = 0x47,
    DW_AT_static_link           = 0x48,
    DW_AT_type                  = 0x49,
    DW_AT_use_location          = 0x4a,
    DW_AT_variable_parameter    = 0x4b,
    DW_AT_virtuality            = 0x4c,
    DW_AT_vtable_elem_location  = 0x4d,
    DW_AT_allocated             = 0x4e,
    DW_AT_associated            = 0x4f,
    DW_AT_data_location         = 0x50,
    DW_AT_byte_stride           = 0x51,
    DW_AT_entry_pc              = 0x52,
    DW_AT_use_UTF8              = 0x53,
    DW_AT_extension             = 0x54,
    DW_AT_ranges                = 0x55,
    DW_AT_trampoline            = 0x56,
    DW_AT_call_column           = 0x57,
    DW_AT_call_file             = 0x58,
    DW_AT_call_line             = 0x59,
    DW_AT_description           = 0x5a,
    DW_AT_binary_scale          = 0x5b,
    DW_AT_decimal_scale         = 0x5c,
    DW_AT_small                 = 0x5d,
    DW_AT_decimal_sign          = 0x5e,
    DW_AT_digit_count           = 0x5f,
    DW_AT_picture_string        = 0x60,
    DW_AT_mutable               = 0x61,
    DW_AT_threads_scaled        = 0x62,
    DW_AT_explicit              = 0x63,
    DW_AT_object_pointer        = 0x64,
    DW_AT_endianity             = 0x65,
    DW_AT_elemental             = 0x66,
    DW_AT_pure                  = 0x67,
    DW_AT_recursive             = 0x68,

    /* Nouveautés v4 */

    DW_AT_signature             = 0x69,
    DW_AT_main_subprogram       = 0x6a,
    DW_AT_data_bit_offset       = 0x6b,
    DW_AT_const_expr            = 0x6c,
    DW_AT_enum_class            = 0x6d,
    DW_AT_linkage_name          = 0x6e,

    DW_AT_lo_user               = 0x2000,
    DW_AT_hi_user               = 0x3fff,

} DwarfAttrib;


#define DW_ATTR_invalid 0 /* FIXME */


typedef struct _dw_value
{
    DwarfAttrib attrib;                     /* Sujet de l'élément          */
    dw_form_value value;                    /* Valeur instanciée associée  */

} dw_value;















#if 0















/* Liste des balises rencontrées */
typedef enum _DwarfTag
{
    DWT_NONE                        = 0x00,

    DWT_ARRAY_TYPE                  = 0x01,
    DWT_CLASS_TYPE                  = 0x02,
    DWT_ENTRY_POINT                 = 0x03,
    DWT_ENUMERATION_TYPE            = 0x04,
    DWT_FORMAL_PARAMETER            = 0x05,
    DWT_IMPORTED_DECLARATION        = 0x08,
    DWT_LABEL                       = 0x0a,
    DWT_LEXICAL_BLOCK               = 0x0b,
    DWT_MEMBER                      = 0x0d,
    DWT_POINTER_TYPE                = 0x0f,
    DWT_REFERENCE_TYPE              = 0x10,
    DWT_COMPILE_UNIT                = 0x11,
    DWT_STRING_TYPE                 = 0x12,
    DWT_STRUCTURE_TYPE              = 0x13,
    DWT_SUBROUTINE_TYPE             = 0x15,
    DWT_TYPEDEF                     = 0x16,
    DWT_UNION_TYPE                  = 0x17,
    DWT_UNSPECIFIED_PARAMETERS      = 0x18,
    DWT_VARIANT                     = 0x19,
    DWT_COMMON_BLOCK                = 0x1a,
    DWT_COMMON_INCLUSION            = 0x1b,
    DWT_INHERITANCE                 = 0x1c,
    DWT_INLINED_SUBROUTINE          = 0x1d,
    DWT_MODULE                      = 0x1e,
    DWT_PTR_TO_MEMBER_TYPE          = 0x1f,
    DWT_SET_TYPE                    = 0x20,
    DWT_SUBRANGE_TYPE               = 0x21,
    DWT_WITH_STMT                   = 0x22,
    DWT_ACCESS_DECLARATION          = 0x23,
    DWT_BASE_TYPE                   = 0x24,
    DWT_CATCH_BLOCK                 = 0x25,
    DWT_CONST_TYPE                  = 0x26,
    DWT_CONSTANT                    = 0x27,
    DWT_ENUMERATOR                  = 0x28,
    DWT_FILE_TYPE                   = 0x29,
    DWT_FRIEND                      = 0x2a,
    DWT_NAMELIST                    = 0x2b,

    DWT_NAMELIST_ITEM               = 0x2c,
    DWT_PACKED_TYPE                 = 0x2d,
    DWT_SUBPROGRAM                  = 0x2e,

    DWT_TEMPLATE_TYPE_PARAM         = 0x2f,     /* DWARF2 */
    DWT_TEMPLATE_TYPE_PARAMETER     = 0x2f,     /* DWARF3 */
    DWT_TEMPLATE_VALUE_PARAM        = 0x30,     /* DWARF2 */
    DWT_TEMPLATE_VALUE_PARAMETER    = 0x30,     /* DWARF3 */
    DWT_THROWN_TYPE                 = 0x31,
    DWT_TRY_BLOCK                   = 0x32,
    DWT_VARIANT_PART                = 0x33,
    DWT_VARIABLE                    = 0x34,
    DWT_VOLATILE_TYPE               = 0x35,
    DWT_DWARF_PROCEDURE             = 0x36,     /* DWARF3 */
    DWT_RESTRICT_TYPE               = 0x37,     /* DWARF3 */
    DWT_INTERFACE_TYPE              = 0x38,     /* DWARF3 */
    DWT_NAMESPACE                   = 0x39,     /* DWARF3 */
    DWT_IMPORTED_MODULE             = 0x3a,     /* DWARF3 */
    DWT_UNSPECIFIED_TYPE            = 0x3b,     /* DWARF3 */
    DWT_PARTIAL_UNIT                = 0x3c,     /* DWARF3 */
    DWT_IMPORTED_UNIT               = 0x3d,     /* DWARF3 */

    DWT_MUTABLE_TYPE                = 0x3e,     /* Oublié en faveur des deux suivantes (DWARF3 -> DWARF3f) */
    DWT_CONDITION                   = 0x3f,     /* DWARF3f */
    DWT_SHARED_TYPE                 = 0x40,     /* DWARF3f */

    DWT_LO_USER                     = 0x4080,

    DWT_MIPS_LOOP                   = 0x4081,

    /* Extensions HP */

    DWT_HP_ARRAY_DESCRIPTOR         = 0x4090,

    /* Extensions GNU */

    DWT_GNU_FORMAT_LABEL            = 0x4101,   /* Fortran. */
    DWT_GNU_FUNCTION_TEMPLATE       = 0x4102,   /* C++ */
    DWT_GNU_CLASS_TEMPLATE          = 0x4103,   /* C++ */
    DWT_GNU_BINCL                   = 0x4104,
    DWT_GNU_EINCL                   = 0x4105,

    /* Extensions ALTIUM */

    DWT_ALTIUM_CIRC_TYPE            = 0x5101,   /* DSP-C/Starcore __circ qualifier */
    DWT_ALTIUM_MWA_CIRC_TYPE        = 0x5102,   /* Starcore __mwa_circ qualifier */        
    DWT_ALTIUM_REV_CARRY_TYPE       = 0x5103,   /* Starcore __rev_carry qualifier */
    DWT_ALTIUM_ROM                  = 0x5111,   /* M16 __rom qualifier */

    /* Extensions pour le support UPC */

    DWT_UPC_SHARED_TYPE             = 0x8765,
    DWT_UPC_STRICT_TYPE             = 0x8766,
    DWT_UPC_RELAXED_TYPE            = 0x8767,

    /* Extensions PGI (STMicroelectronics) */

    DWT_PGI_KANJI_TYPE              = 0xa000,
    DWT_PGI_INTERFACE_BLOCK         = 0xa020,

    /* Extensions SUN */

    DWT_SUN_FUNCTION_TEMPLATE       = 0x4201,
    DWT_SUN_CLASS_TEMPLATE          = 0x4202,
    DWT_SUN_STRUCT_TEMPLATE         = 0x4203,
    DWT_SUN_UNION_TEMPLATE          = 0x4204,
    DWT_SUN_INDIRECT_INHERITANCE    = 0x4205,
    DWT_SUN_CODEFLAGS               = 0x4206,
    DWT_SUN_MEMOP_INFO              = 0x4207,
    DWT_SUN_OMP_CHILD_FUNC          = 0x4208,
    DWT_SUN_RTTI_DESCRIPTOR         = 0x4209,
    DWT_SUN_DTOR_INFO               = 0x420a,
    DWT_SUN_DTOR                    = 0x420b,
    DWT_SUN_F90_INTERFACE           = 0x420c,
    DWT_SUN_FORTRAN_VAX_STRUCTURE   = 0x420d,
    DWT_SUN_HI                      = 0x42ff,

    DWT_HI_USER                     = 0xffff

} DwarfTag;


#define DW_CHILDREN_NO  0x00
#define DW_CHILDREN_YES 0x01


/* Liste des attributs rencontrés */
typedef enum _DwarfAttrib
{
    DWA_SIBLING                         = 0x01,
    DWA_LOCATION                        = 0x02,
    DWA_NAME                            = 0x03,
    DWA_ORDERING                        = 0x09,
    DWA_SUBSCR_DATA                     = 0x0a,
    DWA_BYTE_SIZE                       = 0x0b,
    DWA_BIT_OFFSET                      = 0x0c,
    DWA_BIT_SIZE                        = 0x0d,
    DWA_ELEMENT_LIST                    = 0x0f,
    DWA_STMT_LIST                       = 0x10,
    DWA_LOW_PC                          = 0x11,
    DWA_HIGH_PC                         = 0x12,
    DWA_LANGUAGE                        = 0x13,
    DWA_MEMBER                          = 0x14,
    DWA_DISCR                           = 0x15,
    DWA_DISCR_VALUE                     = 0x16,
    DWA_VISIBILITY                      = 0x17,
    DWA_IMPORT                          = 0x18,
    DWA_STRING_LENGTH                   = 0x19,
    DWA_COMMON_REFERENCE                = 0x1a,
    DWA_COMP_DIR                        = 0x1b,
    DWA_CONST_VALUE                     = 0x1c,
    DWA_CONTAINING_TYPE                 = 0x1d,
    DWA_DEFAULT_VALUE                   = 0x1e,
    DWA_INLINE                          = 0x20,
    DWA_IS_OPTIONAL                     = 0x21,
    DWA_LOWER_BOUND                     = 0x22,
    DWA_PRODUCER                        = 0x25,
    DWA_PROTOTYPED                      = 0x27,
    DWA_RETURN_ADDR                     = 0x2a,
    DWA_START_SCOPE                     = 0x2c,
    DWA_BIT_STRIDE                      = 0x2e,     /* DWARF3 */
    DWA_STRIDE_SIZE                     = 0x2e,     /* DWARF2 */
    DWA_UPPER_BOUND                     = 0x2f,
    DWA_ABSTRACT_ORIGIN                 = 0x31,
    DWA_ACCESSIBILITY                   = 0x32,
    DWA_ADDRESS_CLASS                   = 0x33,
    DWA_ARTIFICIAL                      = 0x34,
    DWA_BASE_TYPES                      = 0x35,
    DWA_CALLING_CONVENTION              = 0x36,
    DWA_COUNT                           = 0x37,
    DWA_DATA_MEMBER_LOCATION            = 0x38,
    DWA_DECL_COLUMN                     = 0x39,
    DWA_DECL_FILE                       = 0x3a,
    DWA_DECL_LINE                       = 0x3b,
    DWA_DECLARATION                     = 0x3c,
    DWA_DISCR_LIST                      = 0x3d,
    DWA_ENCODING                        = 0x3e,
    DWA_EXTERNAL                        = 0x3f,
    DWA_FRAME_BASE                      = 0x40,
    DWA_FRIEND                          = 0x41,
    DWA_IDENTIFIER_CASE                 = 0x42,
    DWA_MACRO_INFO                      = 0x43,
    DWA_NAMELIST_ITEM                   = 0x44,
    DWA_PRIORITY                        = 0x45,
    DWA_SEGMENT                         = 0x46,
    DWA_SPECIFICATION                   = 0x47,
    DWA_STATIC_LINK                     = 0x48,
    DWA_TYPE                            = 0x49,
    DWA_USE_LOCATION                    = 0x4a,
    DWA_VARIABLE_PARAMETER              = 0x4b,
    DWA_VIRTUALITY                      = 0x4c,
    DWA_VTABLE_ELEM_LOCATION            = 0x4d,
    DWA_ALLOCATED                       = 0x4e,     /* DWARF3 */
    DWA_ASSOCIATED                      = 0x4f,     /* DWARF3 */
    DWA_DATA_LOCATION                   = 0x50,     /* DWARF3 */
    DWA_BYTE_STRIDE                     = 0x51,     /* DWARF3f */
    DWA_STRIDE                          = 0x51,     /* DWARF3 (ne pas utiliser) */
    DWA_ENTRY_PC                        = 0x52,     /* DWARF3 */
    DWA_USE_UTF8                        = 0x53,     /* DWARF3 */
    DWA_EXTENSION                       = 0x54,     /* DWARF3 */
    DWA_RANGES                          = 0x55,     /* DWARF3 */
    DWA_TRAMPOLINE                      = 0x56,     /* DWARF3 */
    DWA_CALL_COLUMN                     = 0x57,     /* DWARF3 */
    DWA_CALL_FILE                       = 0x58,     /* DWARF3 */
    DWA_CALL_LINE                       = 0x59,     /* DWARF3 */
    DWA_DESCRIPTION                     = 0x5a,     /* DWARF3 */
    DWA_BINARY_SCALE                    = 0x5b,     /* DWARF3f */
    DWA_DECIMAL_SCALE                   = 0x5c,     /* DWARF3f */
    DWA_SMALL                           = 0x5d,     /* DWARF3f */
    DWA_DECIMAL_SIGN                    = 0x5e,     /* DWARF3f */
    DWA_DIGIT_COUNT                     = 0x5f,     /* DWARF3f */
    DWA_PICTURE_STRING                  = 0x60,     /* DWARF3f */
    DWA_MUTABLE                         = 0x61,     /* DWARF3f */
    DWA_THREADS_SCALED                  = 0x62,     /* DWARF3f */
    DWA_EXPLICIT                        = 0x63,     /* DWARF3f */
    DWA_OBJECT_POINTER                  = 0x64,     /* DWARF3f */
    DWA_ENDIANITY                       = 0x65,     /* DWARF3f */
    DWA_ELEMENTAL                       = 0x66,     /* DWARF3f */
    DWA_PURE                            = 0x67,     /* DWARF3f */
    DWA_RECURSIVE                       = 0x68,     /* DWARF3f */

    DWA_LO_USER                         = 0x2000,

    /* Extension HP */

    DWA_HP_BLOCK_INDEX                  = 0x2000,

    /* Extensions MIPS/SGI */

    DWA_MIPS_FDE                        = 0x2001,
    DWA_MIPS_LOOP_BEGIN                 = 0x2002,
    DWA_MIPS_TAIL_LOOP_BEGIN            = 0x2003,
    DWA_MIPS_EPILOG_BEGIN               = 0x2004,
    DWA_MIPS_LOOP_UNROLL_FACTOR         = 0x2005,
    DWA_MIPS_SOFTWARE_PIPELINE_DEPTH    = 0x2006,
    DWA_MIPS_LINKAGE_NAME               = 0x2007,
    DWA_MIPS_STRIDE                     = 0x2008,
    DWA_MIPS_ABSTRACT_NAME              = 0x2009,
    DWA_MIPS_CLONE_ORIGIN               = 0x200a,
    DWA_MIPS_HAS_INLINES                = 0x200b,
    DWA_MIPS_STRIDE_BYTE                = 0x200c,
    DWA_MIPS_STRIDE_ELEM                = 0x200d,
    DWA_MIPS_PTR_DOPETYPE               = 0x200e,
    DWA_MIPS_ALLOCATABLE_DOPETYPE       = 0x200f,
    DWA_MIPS_ASSUMED_SHAPE_DOPETYPE     = 0x2010,
    DWA_MIPS_ASSUMED_SIZE               = 0x2011,

    /* Extensions HP */

    DWA_HP_unmodifiable                 = 0x2001,   /* conflit : MIPS */
    DWA_HP_actuals_stmt_list            = 0x2010,   /* conflit : MIPS */
    DWA_HP_proc_per_section             = 0x2011,   /* conflit : MIPS */
    DWA_HP_raw_data_ptr                 = 0x2012,
    DWA_HP_pass_by_reference            = 0x2013,
    DWA_HP_opt_level                    = 0x2014,
    DWA_HP_prof_version_id              = 0x2015,
    DWA_HP_opt_flags                    = 0x2016,
    DWA_HP_cold_region_low_pc           = 0x2017,
    DWA_HP_cold_region_high_pc          = 0x2018,
    DWA_HP_all_variables_modifiable     = 0x2019,
    DWA_HP_linkage_name                 = 0x201a,
    DWA_HP_prof_flags                   = 0x201b,

    /* Extensions GNU */

    DWA_GNU_SF_NAMES                    = 0x2101,
    DWA_GNU_SRC_INFO                    = 0x2102,
    DWA_GNU_MAC_INFO                    = 0x2103,
    DWA_GNU_SRC_COORDS                  = 0x2104,
    DWA_GNU_BODY_BEGIN                  = 0x2105,
    DWA_GNU_BODY_END                    = 0x2106,
    DWA_GNU_VECTOR                      = 0x2107,

    /* Extensions SUN */

    DWA_SUN_TEMPLATE                    = 0x2201,
    DWA_SUN_RTNBEG_PD_ADDRESS           = 0x2201,
    DWA_SUN_ALIGNMENT                   = 0x2202,
    DWA_SUN_VTABLE                      = 0x2203,
    DWA_SUN_COUNT_GUARANTEE             = 0x2204,
    DWA_SUN_COMMAND_LINE                = 0x2205,
    DWA_SUN_VBASE                       = 0x2206,
    DWA_SUN_COMPILE_OPTIONS             = 0x2207,
    DWA_SUN_LANGUAGE                    = 0x2208,
    DWA_SUN_BROWSER_FILE                = 0x2209,
    DWA_SUN_VTABLE_ABI                  = 0x2210,
    DWA_SUN_FUNC_OFFSETS                = 0x2211,
    DWA_SUN_CF_KIND                     = 0x2212,
    DWA_SUN_VTABLE_INDEX                = 0x2213,
    DWA_SUN_OMP_TPRIV_ADDR              = 0x2214,
    DWA_SUN_OMP_CHILD_FUNC              = 0x2215,
    DWA_SUN_FUNC_OFFSET                 = 0x2216,
    DWA_SUN_MEMOP_TYPE_REF              = 0x2217,
    DWA_SUN_PROFILE_ID                  = 0x2218,
    DWA_SUN_MEMOP_SIGNATURE             = 0x2219,
    DWA_SUN_OBJ_DIR                     = 0x2220,
    DWA_SUN_OBJ_FILE                    = 0x2221,
    DWA_SUN_ORIGINAL_NAME               = 0x2222,
    DWA_SUN_HWCPROF_SIGNATURE           = 0x2223,
    DWA_SUN_AMD64_PARMDUMP              = 0x2224,
    DWA_SUN_PART_LINK_NAME              = 0x2225,
    DWA_SUN_LINK_NAME                   = 0x2226,
    DWA_SUN_PASS_WITH_CONST             = 0x2227,
    DWA_SUN_RETURN_WITH_CONST           = 0x2228,
    DWA_SUN_IMPORT_BY_NAME              = 0x2229,
    DWA_SUN_F90_POINTER                 = 0x222a,
    DWA_SUN_PASS_BY_REF                 = 0x222b,
    DWA_SUN_F90_ALLOCATABLE             = 0x222c,
    DWA_SUN_F90_ASSUMED_SHAPE_ARRAY     = 0x222d,
    DWA_SUN_C_VLA                       = 0x222e,
    DWA_SUN_RETURN_VALUE_PTR            = 0x2230,
    DWA_SUN_DTOR_START                  = 0x2231,
    DWA_SUN_DTOR_LENGTH                 = 0x2232,
    DWA_SUN_DTOR_STATE_INITIAL          = 0x2233,
    DWA_SUN_DTOR_STATE_FINAL            = 0x2234,
    DWA_SUN_DTOR_STATE_DELTAS           = 0x2235,
    DWA_SUN_IMPORT_BY_LNAME             = 0x2236,
    DWA_SUN_F90_USE_ONLY                = 0x2237,
    DWA_SUN_NAMELIST_SPEC               = 0x2238,
    DWA_SUN_IS_OMP_CHILD_FUNC           = 0x2239,
    DWA_SUN_FORTRAN_MAIN_ALIAS          = 0x223a,
    DWA_SUN_FORTRAN_BASED               = 0x223b,

    /* Extensions ALTIUM */

    DWA_ALTIUM_LOCLIST                  = 0x2300,

    /* Extensions PGI (STMicroelectronics) */

    DWA_PGI_LBASE                       = 0x3a00,
    DWA_PGI_SOFFSET                     = 0x3a01,
    DWA_PGI_LSTRIDE                     = 0x3a02,

    /* Extensions pour le support UPC */

    DWA_UPC_THREADS_SCALED              = 0x3210,

    DWA_HI_USER                         = 0x3fff

} DwarfAttrib;


/* Liste des types de données */
typedef enum _DwarfForm
{
    DWF_ADDR        = 0x01,
    DWF_BLOCK2      = 0x03,
    DWF_BLOCK4      = 0x04,
    DWF_DATA2       = 0x05,
    DWF_DATA4       = 0x06,
    DWF_DATA8       = 0x07,
    DWF_STRING      = 0x08,
    DWF_BLOCK       = 0x09,
    DWF_BLOCK1      = 0x0a,
    DWF_DATA1       = 0x0b,
    DWF_FLAG        = 0x0c,
    DWF_SDATA       = 0x0d,
    DWF_STRP        = 0x0e,
    DWF_UDATA       = 0x0f,
    DWF_REF_ADDR    = 0x10,
    DWF_REF1        = 0x11,
    DWF_REF2        = 0x12,
    DWF_REF4        = 0x13,
    DWF_REF8        = 0x14,
    DWF_REF_UDATA   = 0x15,
    DWF_INDIRECT    = 0x16

} DwarfForm;


#endif



#endif  /* _FORMAT_DWARF_DWARF_DEF_H */
